@@ -3,9 +3,9 @@ package HTML::Mason;
 # This program is free software; you can redistribute it and/or modify it
 # under the same terms as Perl itself.
 
-use 5.006_000;
+use 5.006;
 
-$HTML::Mason::VERSION = '1.3101';
+$HTML::Mason::VERSION = '1.32';
 
 use HTML::Mason::Interp;
 
